const coral = @import("../coral.zig");

const std = @import("std");

identifier: [:0]const u8,
type: *const coral.shaders.Type,
has_next: ?*const Self = null,

const Self = @This();

pub fn fieldsOf(comptime Aggregate: type) ?*const Self {
    return switch (@typeInfo(Aggregate)) {
        .@"union" => |@"union"| unionFields(@"union".fields),

        .@"struct" => |@"struct"| switch (@"struct".is_tuple) {
            false => structFields(@"struct".fields),
            true => @compileError("tuple types are unsupported as their fields are unidentifierd"),
        },

        else => @compileError(std.fmt.comptimePrint("non-aggregate types like {s} are unsupported as they do not contain fields", .{
            @typeName(Aggregate),
        })),
    };
}

pub fn has(self: *const Self, identifier: []const u8) ?*const Self {
    var field = self;

    while (true) {
        if (std.mem.eql(u8, field.identifier, identifier)) {
            return field;
        }

        field = field.has_next orelse {
            return null;
        };
    }
}

fn structFields(comptime fields: []const std.builtin.Type.StructField) ?*const Self {
    if (fields.len != 0) {
        const field = fields[0];

        const struct_fields = struct {
            const instance = Self{
                .identifier = field.name,
                .type = typeOf(field.type),
                .has_next = structFields(fields[1..]),
            };
        };

        return &struct_fields.instance;
    }

    return null;
}

pub fn typeOf(comptime FieldType: type) *const coral.shaders.Type {
    return switch (FieldType) {
        i32 => .int,
        f32 => .float,
        else => @compileError(std.fmt.comptimePrint("{s} is not a valid field type", .{@typeName(FieldType)})),
    };
}

fn unionFields(comptime fields: []const std.builtin.Type.UnionField) ?*const Self {
    if (fields.len != 0) {
        const field = fields[0];

        const uniform_fields = struct {
            const instance = Self{
                .identifier = field.name,
                .type = typeOf(field.type),
                .has_next = structFields(fields[1..]),
            };
        };

        return &uniform_fields.instance;
    }

    return null;
}

pub const vector_x = &Self{
    .identifier = "x",
    .type = .float,
};

pub const vector_y = &Self{
    .identifier = "y",
    .type = .float,
};

pub const vector_z = &Self{
    .identifier = "z",
    .type = .float,
};

pub const vector_w = &Self{
    .identifier = "z",
    .type = .float,
};

pub const vector_xx = &Self{
    .identifier = "xx",
    .type = .float2,
};

pub const vector_xy = &Self{
    .identifier = "xy",
    .type = .float2,
};

pub const vector_xz = &Self{
    .identifier = "xz",
    .type = .float2,
};

pub const vector_xw = &Self{
    .identifier = "xz",
    .type = .float2,
};

pub const vector_yx = &Self{
    .identifier = "yx",
    .type = .float2,
};

pub const vector_yy = &Self{
    .identifier = "yy",
    .type = .float2,
};

pub const vector_yz = &Self{
    .identifier = "yz",
    .type = .float2,
};

pub const vector_yw = &Self{
    .identifier = "yz",
    .type = .float2,
};

pub const vector_zx = &Self{
    .identifier = "zx",
    .type = .float2,
};

pub const vector_zy = &Self{
    .identifier = "zy",
    .type = .float2,
};

pub const vector_zz = &Self{
    .identifier = "zz",
    .type = .float2,
};

pub const vector_zw = &Self{
    .identifier = "zw",
    .type = .float2,
};

pub const vector_wx = &Self{
    .identifier = "wx",
    .type = .float2,
};

pub const vector_wy = &Self{
    .identifier = "wy",
    .type = .float2,
};

pub const vector_wz = &Self{
    .identifier = "wz",
    .type = .float2,
};

pub const vector_ww = &Self{
    .identifier = "ww",
    .type = .float2,
};

pub const vector_xxx = &Self{
    .identifier = "xxx",
    .type = .float3,
};

pub const vector_xyx = &Self{
    .identifier = "xyx",
    .type = .float3,
};

pub const vector_xzx = &Self{
    .identifier = "xzx",
    .type = .float3,
};

pub const vector_xwx = &Self{
    .identifier = "xwx",
    .type = .float3,
};

pub const vector_yxx = &Self{
    .identifier = "yxx",
    .type = .float3,
};

pub const vector_yyx = &Self{
    .identifier = "yyx",
    .type = .float3,
};

pub const vector_yzx = &Self{
    .identifier = "yzx",
    .type = .float3,
};

pub const vector_ywx = &Self{
    .identifier = "ywx",
    .type = .float3,
};

pub const vector_zxx = &Self{
    .identifier = "zxx",
    .type = .float3,
};

pub const vector_zyx = &Self{
    .identifier = "zyx",
    .type = .float3,
};

pub const vector_zzx = &Self{
    .identifier = "zzx",
    .type = .float3,
};

pub const vector_zwx = &Self{
    .identifier = "zwx",
    .type = .float3,
};

pub const vector_wxx = &Self{
    .identifier = "wxx",
    .type = .float3,
};

pub const vector_wyx = &Self{
    .identifier = "wyy",
    .type = .float3,
};

pub const vector_wzx = &Self{
    .identifier = "wzx",
    .type = .float3,
};

pub const vector_wwx = &Self{
    .identifier = "wwx",
    .type = .float3,
};

pub const vector_xxy = &Self{
    .identifier = "xxy",
    .type = .float3,
};

pub const vector_xyy = &Self{
    .identifier = "xyy",
    .type = .float3,
};

pub const vector_xzy = &Self{
    .identifier = "xzy",
    .type = .float3,
};

pub const vector_xwy = &Self{
    .identifier = "xwy",
    .type = .float3,
};

pub const vector_yxy = &Self{
    .identifier = "yxy",
    .type = .float3,
};

pub const vector_yyy = &Self{
    .identifier = "yyy",
    .type = .float3,
};

pub const vector_yzy = &Self{
    .identifier = "yzy",
    .type = .float3,
};

pub const vector_ywy = &Self{
    .identifier = "ywy",
    .type = .float3,
};

pub const vector_zxy = &Self{
    .identifier = "zxy",
    .type = .float3,
};

pub const vector_zyy = &Self{
    .identifier = "zyy",
    .type = .float3,
};

pub const vector_zzy = &Self{
    .identifier = "zzy",
    .type = .float3,
};

pub const vector_zwy = &Self{
    .identifier = "zwy",
    .type = .float3,
};

pub const vector_wxy = &Self{
    .identifier = "wxy",
    .type = .float3,
};

pub const vector_wyy = &Self{
    .identifier = "wyy",
    .type = .float3,
};

pub const vector_wzy = &Self{
    .identifier = "wzy",
    .type = .float3,
};

pub const vector_wwy = &Self{
    .identifier = "wwy",
    .type = .float3,
};

pub const vector_xxz = &Self{
    .identifier = "xxz",
    .type = .float3,
};

pub const vector_xyz = &Self{
    .identifier = "xyz",
    .type = .float3,
};

pub const vector_xzz = &Self{
    .identifier = "xzz",
    .type = .float3,
};

pub const vector_xwz = &Self{
    .identifier = "xwz",
    .type = .float3,
};

pub const vector_yxz = &Self{
    .identifier = "yxz",
    .type = .float3,
};

pub const vector_yyz = &Self{
    .identifier = "yyz",
    .type = .float3,
};

pub const vector_yzz = &Self{
    .identifier = "yzz",
    .type = .float3,
};

pub const vector_ywz = &Self{
    .identifier = "ywz",
    .type = .float3,
};

pub const vector_zxz = &Self{
    .identifier = "zxz",
    .type = .float3,
};

pub const vector_zyz = &Self{
    .identifier = "zyz",
    .type = .float3,
};

pub const vector_zzz = &Self{
    .identifier = "zzz",
    .type = .float3,
};

pub const vector_zwz = &Self{
    .identifier = "zwz",
    .type = .float3,
};

pub const vector_wxz = &Self{
    .identifier = "wxz",
    .type = .float3,
};

pub const vector_wyz = &Self{
    .identifier = "wyz",
    .type = .float3,
};

pub const vector_wzz = &Self{
    .identifier = "wzz",
    .type = .float3,
};

pub const vector_wwz = &Self{
    .identifier = "wwz",
    .type = .float3,
};

pub const vector_xxw = &Self{
    .identifier = "xxw",
    .type = .float3,
};

pub const vector_xyw = &Self{
    .identifier = "xyw",
    .type = .float3,
};

pub const vector_xzw = &Self{
    .identifier = "xzw",
    .type = .float3,
};

pub const vector_xww = &Self{
    .identifier = "xww",
    .type = .float3,
};

pub const vector_yxw = &Self{
    .identifier = "yxw",
    .type = .float3,
};

pub const vector_yyw = &Self{
    .identifier = "yyw",
    .type = .float3,
};

pub const vector_yzw = &Self{
    .identifier = "yzw",
    .type = .float3,
};

pub const vector_yww = &Self{
    .identifier = "yww",
    .type = .float3,
};

pub const vector_zxw = &Self{
    .identifier = "zxw",
    .type = .float3,
};

pub const vector_zyw = &Self{
    .identifier = "zyw",
    .type = .float3,
};

pub const vector_zzw = &Self{
    .identifier = "zzw",
    .type = .float3,
};

pub const vector_zww = &Self{
    .identifier = "zww",
    .type = .float3,
};

pub const vector_wxw = &Self{
    .identifier = "wxw",
    .type = .float3,
};

pub const vector_wyw = &Self{
    .identifier = "wyw",
    .type = .float3,
};

pub const vector_wzw = &Self{
    .identifier = "wzw",
    .type = .float3,
};

pub const vector_www = &Self{
    .identifier = "www",
    .type = .float3,
};

pub const vector_xxxx = &Self{
    .identifier = "xxxx",
    .type = .float4,
};

pub const vector_xxxy = &Self{
    .identifier = "xxxy",
    .type = .float4,
};

pub const vector_xxxz = &Self{
    .identifier = "xxxz",
    .type = .float4,
};

pub const vector_xxxw = &Self{
    .identifier = "xxxw",
    .type = .float4,
};

pub const vector_xxyx = &Self{
    .identifier = "xxyx",
    .type = .float4,
};

pub const vector_xxyy = &Self{
    .identifier = "xxyy",
    .type = .float4,
};

pub const vector_xxyz = &Self{
    .identifier = "xxyz",
    .type = .float4,
};

pub const vector_xxyw = &Self{
    .identifier = "xxyw",
    .type = .float4,
};

pub const vector_xxzx = &Self{
    .identifier = "xxzx",
    .type = .float4,
};

pub const vector_xxzy = &Self{
    .identifier = "xxzy",
    .type = .float4,
};

pub const vector_xxzz = &Self{
    .identifier = "xxzz",
    .type = .float4,
};

pub const vector_xxzw = &Self{
    .identifier = "xxzw",
    .type = .float4,
};

pub const vector_xxwx = &Self{
    .identifier = "xxwx",
    .type = .float4,
};

pub const vector_xxwy = &Self{
    .identifier = "xxwy",
    .type = .float4,
};

pub const vector_xxwz = &Self{
    .identifier = "xxwz",
    .type = .float4,
};

pub const vector_xxww = &Self{
    .identifier = "xxww",
    .type = .float4,
};

pub const vector_xyxx = &Self{
    .identifier = "xyxx",
    .type = .float4,
};

pub const vector_xyxy = &Self{
    .identifier = "xyxy",
    .type = .float4,
};

pub const vector_xyxz = &Self{
    .identifier = "xyxz",
    .type = .float4,
};

pub const vector_xyxw = &Self{
    .identifier = "xyxw",
    .type = .float4,
};

pub const vector_xyyx = &Self{
    .identifier = "xyyx",
    .type = .float4,
};

pub const vector_xyyy = &Self{
    .identifier = "xyyy",
    .type = .float4,
};

pub const vector_xyyz = &Self{
    .identifier = "xyyz",
    .type = .float4,
};

pub const vector_xyyw = &Self{
    .identifier = "xyyw",
    .type = .float4,
};

pub const vector_xyzx = &Self{
    .identifier = "xyzx",
    .type = .float4,
};

pub const vector_xyzy = &Self{
    .identifier = "xyzy",
    .type = .float4,
};

pub const vector_xyzz = &Self{
    .identifier = "xyzz",
    .type = .float4,
};

pub const vector_xyzw = &Self{
    .identifier = "xyzw",
    .type = .float4,
};

pub const vector_xywx = &Self{
    .identifier = "xywx",
    .type = .float4,
};

pub const vector_xywy = &Self{
    .identifier = "xywy",
    .type = .float4,
};

pub const vector_xywz = &Self{
    .identifier = "xywz",
    .type = .float4,
};

pub const vector_xyww = &Self{
    .identifier = "xyww",
    .type = .float4,
};

pub const vector_xzxx = &Self{
    .identifier = "xzxx",
    .type = .float4,
};

pub const vector_xzxy = &Self{
    .identifier = "xzxy",
    .type = .float4,
};

pub const vector_xzxz = &Self{
    .identifier = "xzxz",
    .type = .float4,
};

pub const vector_xzxw = &Self{
    .identifier = "xzxw",
    .type = .float4,
};

pub const vector_xzyx = &Self{
    .identifier = "xzyx",
    .type = .float4,
};

pub const vector_xzyy = &Self{
    .identifier = "xzyy",
    .type = .float4,
};

pub const vector_xzyz = &Self{
    .identifier = "xzyz",
    .type = .float4,
};

pub const vector_xzyw = &Self{
    .identifier = "xzyw",
    .type = .float4,
};

pub const vector_xzzx = &Self{
    .identifier = "xzzx",
    .type = .float4,
};

pub const vector_xzzy = &Self{
    .identifier = "xzzy",
    .type = .float4,
};

pub const vector_xzzz = &Self{
    .identifier = "xzzz",
    .type = .float4,
};

pub const vector_xzzw = &Self{
    .identifier = "xzzw",
    .type = .float4,
};

pub const vector_xzwx = &Self{
    .identifier = "xzwx",
    .type = .float4,
};

pub const vector_xzwy = &Self{
    .identifier = "xzwy",
    .type = .float4,
};

pub const vector_xzwz = &Self{
    .identifier = "xzwz",
    .type = .float4,
};

pub const vector_xzww = &Self{
    .identifier = "xzww",
    .type = .float4,
};

pub const vector_xwxx = &Self{
    .identifier = "xwxx",
    .type = .float4,
};

pub const vector_xwxy = &Self{
    .identifier = "xwxy",
    .type = .float4,
};

pub const vector_xwxz = &Self{
    .identifier = "xwxz",
    .type = .float4,
};

pub const vector_xwxw = &Self{
    .identifier = "xwxw",
    .type = .float4,
};

pub const vector_xwyx = &Self{
    .identifier = "xwyx",
    .type = .float4,
};

pub const vector_xwyy = &Self{
    .identifier = "xwyy",
    .type = .float4,
};

pub const vector_xwyz = &Self{
    .identifier = "xwyz",
    .type = .float4,
};

pub const vector_xwyw = &Self{
    .identifier = "xwyw",
    .type = .float4,
};

pub const vector_xwzx = &Self{
    .identifier = "xwzx",
    .type = .float4,
};

pub const vector_xwzy = &Self{
    .identifier = "xwzy",
    .type = .float4,
};

pub const vector_xwzz = &Self{
    .identifier = "xwzz",
    .type = .float4,
};

pub const vector_xwzw = &Self{
    .identifier = "xwzw",
    .type = .float4,
};

pub const vector_xwwx = &Self{
    .identifier = "xwwx",
    .type = .float4,
};

pub const vector_xwwy = &Self{
    .identifier = "xwwy",
    .type = .float4,
};

pub const vector_xwwz = &Self{
    .identifier = "xwwz",
    .type = .float4,
};

pub const vector_xwww = &Self{
    .identifier = "xwww",
    .type = .float4,
};

pub const vector_wxxx = &Self{
    .identifier = "wxxx",
    .type = .float4,
};

pub const vector_wxxy = &Self{
    .identifier = "wxxy",
    .type = .float4,
};

pub const vector_wxxz = &Self{
    .identifier = "wxxz",
    .type = .float4,
};

pub const vector_wxxw = &Self{
    .identifier = "wxxw",
    .type = .float4,
};

pub const vector_wxyx = &Self{
    .identifier = "wxyx",
    .type = .float4,
};

pub const vector_wxyy = &Self{
    .identifier = "wxyy",
    .type = .float4,
};

pub const vector_wxyz = &Self{
    .identifier = "wxyz",
    .type = .float4,
};

pub const vector_wxyw = &Self{
    .identifier = "wxyw",
    .type = .float4,
};

pub const vector_wxzx = &Self{
    .identifier = "wxzx",
    .type = .float4,
};

pub const vector_wxzy = &Self{
    .identifier = "wxzy",
    .type = .float4,
};

pub const vector_wxzz = &Self{
    .identifier = "wxzz",
    .type = .float4,
};

pub const vector_wxzw = &Self{
    .identifier = "wxzw",
    .type = .float4,
};

pub const vector_wxwx = &Self{
    .identifier = "wxwx",
    .type = .float4,
};

pub const vector_wxwy = &Self{
    .identifier = "wxwy",
    .type = .float4,
};

pub const vector_wxwz = &Self{
    .identifier = "wxwz",
    .type = .float4,
};

pub const vector_wxww = &Self{
    .identifier = "wxww",
    .type = .float4,
};

pub const vector_wyxx = &Self{
    .identifier = "wyxx",
    .type = .float4,
};

pub const vector_wyxy = &Self{
    .identifier = "wyxy",
    .type = .float4,
};

pub const vector_wyxz = &Self{
    .identifier = "wyxz",
    .type = .float4,
};

pub const vector_wyxw = &Self{
    .identifier = "wyxw",
    .type = .float4,
};

pub const vector_wyyx = &Self{
    .identifier = "wyyx",
    .type = .float4,
};

pub const vector_wyyy = &Self{
    .identifier = "wyyy",
    .type = .float4,
};

pub const vector_wyyz = &Self{
    .identifier = "wyyz",
    .type = .float4,
};

pub const vector_wyyw = &Self{
    .identifier = "wyyw",
    .type = .float4,
};

pub const vector_wyzx = &Self{
    .identifier = "wyzx",
    .type = .float4,
};

pub const vector_wyzy = &Self{
    .identifier = "wyzy",
    .type = .float4,
};

pub const vector_wyzz = &Self{
    .identifier = "wyzz",
    .type = .float4,
};

pub const vector_wyzw = &Self{
    .identifier = "wyzw",
    .type = .float4,
};

pub const vector_wywx = &Self{
    .identifier = "wywx",
    .type = .float4,
};

pub const vector_wywy = &Self{
    .identifier = "wywy",
    .type = .float4,
};

pub const vector_wywz = &Self{
    .identifier = "wywz",
    .type = .float4,
};

pub const vector_wyww = &Self{
    .identifier = "wyww",
    .type = .float4,
};

pub const vector_wzxx = &Self{
    .identifier = "wzxx",
    .type = .float4,
};

pub const vector_wzxy = &Self{
    .identifier = "wzxy",
    .type = .float4,
};

pub const vector_wzxz = &Self{
    .identifier = "wzxz",
    .type = .float4,
};

pub const vector_wzxw = &Self{
    .identifier = "wzxw",
    .type = .float4,
};

pub const vector_wzyx = &Self{
    .identifier = "wzyx",
    .type = .float4,
};

pub const vector_wzyy = &Self{
    .identifier = "wzyy",
    .type = .float4,
};

pub const vector_wzyz = &Self{
    .identifier = "wzyz",
    .type = .float4,
};

pub const vector_wzyw = &Self{
    .identifier = "wzyw",
    .type = .float4,
};

pub const vector_wzzx = &Self{
    .identifier = "wzzx",
    .type = .float4,
};

pub const vector_wzzy = &Self{
    .identifier = "wzzy",
    .type = .float4,
};

pub const vector_wzzz = &Self{
    .identifier = "wzzz",
    .type = .float4,
};

pub const vector_wzzw = &Self{
    .identifier = "wzzw",
    .type = .float4,
};

pub const vector_wzwx = &Self{
    .identifier = "wzwx",
    .type = .float4,
};

pub const vector_wzwy = &Self{
    .identifier = "wzwy",
    .type = .float4,
};

pub const vector_wzwz = &Self{
    .identifier = "wzwz",
    .type = .float4,
};

pub const vector_wzww = &Self{
    .identifier = "wzww",
    .type = .float4,
};

pub const vector_wwxx = &Self{
    .identifier = "wwxx",
    .type = .float4,
};

pub const vector_wwxy = &Self{
    .identifier = "wwxy",
    .type = .float4,
};

pub const vector_wwxz = &Self{
    .identifier = "wwxz",
    .type = .float4,
};

pub const vector_wwxw = &Self{
    .identifier = "wwxw",
    .type = .float4,
};

pub const vector_wwyx = &Self{
    .identifier = "wwyx",
    .type = .float4,
};

pub const vector_wwyy = &Self{
    .identifier = "wwyy",
    .type = .float4,
};

pub const vector_wwyz = &Self{
    .identifier = "wwyz",
    .type = .float4,
};

pub const vector_wwyw = &Self{
    .identifier = "wwyw",
    .type = .float4,
};

pub const vector_wwzx = &Self{
    .identifier = "wwzx",
    .type = .float4,
};

pub const vector_wwzy = &Self{
    .identifier = "wwzy",
    .type = .float4,
};

pub const vector_wwzz = &Self{
    .identifier = "wwzz",
    .type = .float4,
};

pub const vector_wwzw = &Self{
    .identifier = "wwzw",
    .type = .float4,
};

pub const vector_wwwx = &Self{
    .identifier = "wwwx",
    .type = .float4,
};

pub const vector_wwwy = &Self{
    .identifier = "wwwy",
    .type = .float4,
};

pub const vector_wwwz = &Self{
    .identifier = "wwwz",
    .type = .float4,
};

pub const vector_wwww = &Self{
    .identifier = "wwww",
    .type = .float4,
};

pub const vector_yxxx = &Self{
    .identifier = "yxxx",
    .type = .float4,
};

pub const vector_yxxy = &Self{
    .identifier = "yxxy",
    .type = .float4,
};

pub const vector_yxxz = &Self{
    .identifier = "yxxz",
    .type = .float4,
};

pub const vector_yxxw = &Self{
    .identifier = "yxxw",
    .type = .float4,
};

pub const vector_yxyx = &Self{
    .identifier = "yxyx",
    .type = .float4,
};

pub const vector_yxyy = &Self{
    .identifier = "yxyy",
    .type = .float4,
};

pub const vector_yxyz = &Self{
    .identifier = "yxyz",
    .type = .float4,
};

pub const vector_yxyw = &Self{
    .identifier = "yxyw",
    .type = .float4,
};

pub const vector_yxzx = &Self{
    .identifier = "yxzx",
    .type = .float4,
};

pub const vector_yxzy = &Self{
    .identifier = "yxzy",
    .type = .float4,
};

pub const vector_yxzz = &Self{
    .identifier = "yxzz",
    .type = .float4,
};

pub const vector_yxzw = &Self{
    .identifier = "yxzw",
    .type = .float4,
};

pub const vector_yxwx = &Self{
    .identifier = "yxwx",
    .type = .float4,
};

pub const vector_yxwy = &Self{
    .identifier = "yxwy",
    .type = .float4,
};

pub const vector_yxwz = &Self{
    .identifier = "yxwz",
    .type = .float4,
};

pub const vector_yxww = &Self{
    .identifier = "yxww",
    .type = .float4,
};

pub const vector_yyxx = &Self{
    .identifier = "yyxx",
    .type = .float4,
};

pub const vector_yyxy = &Self{
    .identifier = "yyxy",
    .type = .float4,
};

pub const vector_yyxz = &Self{
    .identifier = "yyxz",
    .type = .float4,
};

pub const vector_yyxw = &Self{
    .identifier = "yyxw",
    .type = .float4,
};

pub const vector_yyyx = &Self{
    .identifier = "yyyx",
    .type = .float4,
};

pub const vector_yyyy = &Self{
    .identifier = "yyyy",
    .type = .float4,
};

pub const vector_yyyz = &Self{
    .identifier = "yyyz",
    .type = .float4,
};

pub const vector_yyyw = &Self{
    .identifier = "yyyw",
    .type = .float4,
};

pub const vector_yyzx = &Self{
    .identifier = "yyzx",
    .type = .float4,
};

pub const vector_yyzy = &Self{
    .identifier = "yyzy",
    .type = .float4,
};

pub const vector_yyzz = &Self{
    .identifier = "yyzz",
    .type = .float4,
};

pub const vector_yyzw = &Self{
    .identifier = "yyzw",
    .type = .float4,
};

pub const vector_yywx = &Self{
    .identifier = "yywx",
    .type = .float4,
};

pub const vector_yywy = &Self{
    .identifier = "yywy",
    .type = .float4,
};

pub const vector_yywz = &Self{
    .identifier = "yywz",
    .type = .float4,
};

pub const vector_yyww = &Self{
    .identifier = "yyww",
    .type = .float4,
};

pub const vector_yzxx = &Self{
    .identifier = "yzxx",
    .type = .float4,
};

pub const vector_yzxy = &Self{
    .identifier = "yzxy",
    .type = .float4,
};

pub const vector_yzxz = &Self{
    .identifier = "yzxz",
    .type = .float4,
};

pub const vector_yzxw = &Self{
    .identifier = "yzxw",
    .type = .float4,
};

pub const vector_yzyx = &Self{
    .identifier = "yzyx",
    .type = .float4,
};

pub const vector_yzyy = &Self{
    .identifier = "yzyy",
    .type = .float4,
};

pub const vector_yzyz = &Self{
    .identifier = "yzyz",
    .type = .float4,
};

pub const vector_yzyw = &Self{
    .identifier = "yzyw",
    .type = .float4,
};

pub const vector_yzzx = &Self{
    .identifier = "yzzx",
    .type = .float4,
};

pub const vector_yzzy = &Self{
    .identifier = "yzzy",
    .type = .float4,
};

pub const vector_yzzz = &Self{
    .identifier = "yzzz",
    .type = .float4,
};

pub const vector_yzzw = &Self{
    .identifier = "yzzw",
    .type = .float4,
};

pub const vector_yzwx = &Self{
    .identifier = "yzwx",
    .type = .float4,
};

pub const vector_yzwy = &Self{
    .identifier = "yzwy",
    .type = .float4,
};

pub const vector_yzwz = &Self{
    .identifier = "yzwz",
    .type = .float4,
};

pub const vector_yzww = &Self{
    .identifier = "yzww",
    .type = .float4,
};

pub const vector_ywxx = &Self{
    .identifier = "ywxx",
    .type = .float4,
};

pub const vector_ywxy = &Self{
    .identifier = "ywxy",
    .type = .float4,
};

pub const vector_ywxz = &Self{
    .identifier = "ywxz",
    .type = .float4,
};

pub const vector_ywxw = &Self{
    .identifier = "ywxw",
    .type = .float4,
};

pub const vector_ywyx = &Self{
    .identifier = "ywyx",
    .type = .float4,
};

pub const vector_ywyy = &Self{
    .identifier = "ywyy",
    .type = .float4,
};

pub const vector_ywyz = &Self{
    .identifier = "ywyz",
    .type = .float4,
};

pub const vector_ywyw = &Self{
    .identifier = "ywyw",
    .type = .float4,
};

pub const vector_ywzx = &Self{
    .identifier = "ywzx",
    .type = .float4,
};

pub const vector_ywzy = &Self{
    .identifier = "ywzy",
    .type = .float4,
};

pub const vector_ywzz = &Self{
    .identifier = "ywzz",
    .type = .float4,
};

pub const vector_ywzw = &Self{
    .identifier = "ywzw",
    .type = .float4,
};

pub const vector_ywwx = &Self{
    .identifier = "ywwx",
    .type = .float4,
};

pub const vector_ywwy = &Self{
    .identifier = "ywwy",
    .type = .float4,
};

pub const vector_ywwz = &Self{
    .identifier = "ywwz",
    .type = .float4,
};

pub const vector_ywww = &Self{
    .identifier = "ywww",
    .type = .float4,
};

pub const vector_zxxx = &Self{
    .identifier = "zxxx",
    .type = .float4,
};

pub const vector_zxxy = &Self{
    .identifier = "zxxy",
    .type = .float4,
};

pub const vector_zxxz = &Self{
    .identifier = "zxxz",
    .type = .float4,
};

pub const vector_zxxw = &Self{
    .identifier = "zxxw",
    .type = .float4,
};

pub const vector_zxyx = &Self{
    .identifier = "zxyx",
    .type = .float4,
};

pub const vector_zxyy = &Self{
    .identifier = "zxyy",
    .type = .float4,
};

pub const vector_zxyz = &Self{
    .identifier = "zxyz",
    .type = .float4,
};

pub const vector_zxyw = &Self{
    .identifier = "zxyw",
    .type = .float4,
};

pub const vector_zxzx = &Self{
    .identifier = "zxzx",
    .type = .float4,
};

pub const vector_zxzy = &Self{
    .identifier = "zxzy",
    .type = .float4,
};

pub const vector_zxzz = &Self{
    .identifier = "zxzz",
    .type = .float4,
};

pub const vector_zxzw = &Self{
    .identifier = "zxzw",
    .type = .float4,
};

pub const vector_zxwx = &Self{
    .identifier = "zxwx",
    .type = .float4,
};

pub const vector_zxwy = &Self{
    .identifier = "zxwy",
    .type = .float4,
};

pub const vector_zxwz = &Self{
    .identifier = "zxwz",
    .type = .float4,
};

pub const vector_zxww = &Self{
    .identifier = "zxww",
    .type = .float4,
};

pub const vector_zyxx = &Self{
    .identifier = "zyxx",
    .type = .float4,
};

pub const vector_zyxy = &Self{
    .identifier = "zyxy",
    .type = .float4,
};

pub const vector_zyxz = &Self{
    .identifier = "zyxz",
    .type = .float4,
};

pub const vector_zyxw = &Self{
    .identifier = "zyxw",
    .type = .float4,
};

pub const vector_zyyx = &Self{
    .identifier = "zyyx",
    .type = .float4,
};

pub const vector_zyyy = &Self{
    .identifier = "zyyy",
    .type = .float4,
};

pub const vector_zyyz = &Self{
    .identifier = "zyyz",
    .type = .float4,
};

pub const vector_zyyw = &Self{
    .identifier = "zyyw",
    .type = .float4,
};

pub const vector_zyzx = &Self{
    .identifier = "zyzx",
    .type = .float4,
};

pub const vector_zyzy = &Self{
    .identifier = "zyzy",
    .type = .float4,
};

pub const vector_zyzz = &Self{
    .identifier = "zyzz",
    .type = .float4,
};

pub const vector_zyzw = &Self{
    .identifier = "zyzw",
    .type = .float4,
};

pub const vector_zywx = &Self{
    .identifier = "zywx",
    .type = .float4,
};

pub const vector_zywy = &Self{
    .identifier = "zywy",
    .type = .float4,
};

pub const vector_zywz = &Self{
    .identifier = "zywz",
    .type = .float4,
};

pub const vector_zyww = &Self{
    .identifier = "zyww",
    .type = .float4,
};

pub const vector_zzxx = &Self{
    .identifier = "zzxx",
    .type = .float4,
};

pub const vector_zzxy = &Self{
    .identifier = "zzxy",
    .type = .float4,
};

pub const vector_zzxz = &Self{
    .identifier = "zzxz",
    .type = .float4,
};

pub const vector_zzxw = &Self{
    .identifier = "zzxw",
    .type = .float4,
};

pub const vector_zzyx = &Self{
    .identifier = "zzyx",
    .type = .float4,
};

pub const vector_zzyy = &Self{
    .identifier = "zzyy",
    .type = .float4,
};

pub const vector_zzyz = &Self{
    .identifier = "zzyz",
    .type = .float4,
};

pub const vector_zzyw = &Self{
    .identifier = "zzyw",
    .type = .float4,
};

pub const vector_zzzx = &Self{
    .identifier = "zzzx",
    .type = .float4,
};

pub const vector_zzzy = &Self{
    .identifier = "zzzy",
    .type = .float4,
};

pub const vector_zzzz = &Self{
    .identifier = "zzzz",
    .type = .float4,
};

pub const vector_zzzw = &Self{
    .identifier = "zzzw",
    .type = .float4,
};

pub const vector_zzwx = &Self{
    .identifier = "zzwx",
    .type = .float4,
};

pub const vector_zzwy = &Self{
    .identifier = "zzwy",
    .type = .float4,
};

pub const vector_zzwz = &Self{
    .identifier = "zzwz",
    .type = .float4,
};

pub const vector_zzww = &Self{
    .identifier = "zzww",
    .type = .float4,
};

pub const vector_zwxx = &Self{
    .identifier = "zwxx",
    .type = .float4,
};

pub const vector_zwxy = &Self{
    .identifier = "zwxy",
    .type = .float4,
};

pub const vector_zwxz = &Self{
    .identifier = "zwxz",
    .type = .float4,
};

pub const vector_zwxw = &Self{
    .identifier = "zwxw",
    .type = .float4,
};

pub const vector_zwyx = &Self{
    .identifier = "zwyx",
    .type = .float4,
};

pub const vector_zwyy = &Self{
    .identifier = "zwyy",
    .type = .float4,
};

pub const vector_zwyz = &Self{
    .identifier = "zwyz",
    .type = .float4,
};

pub const vector_zwyw = &Self{
    .identifier = "zwyw",
    .type = .float4,
};

pub const vector_zwzx = &Self{
    .identifier = "zwzx",
    .type = .float4,
};

pub const vector_zwzy = &Self{
    .identifier = "zwzy",
    .type = .float4,
};

pub const vector_zwzz = &Self{
    .identifier = "zwzz",
    .type = .float4,
};

pub const vector_zwzw = &Self{
    .identifier = "zwzw",
    .type = .float4,
};

pub const vector_zwwx = &Self{
    .identifier = "zwwx",
    .type = .float4,
};

pub const vector_zwwy = &Self{
    .identifier = "zwwy",
    .type = .float4,
};

pub const vector_zwwz = &Self{
    .identifier = "zwwz",
    .type = .float4,
};

pub const vector_zwww = &Self{
    .identifier = "zwww",
    .type = .float4,
};
