const coral = @import("../coral.zig");

identifier: [:0]const u8,
layout: Layout,

pub const Layout = union(enum) {
    handle,
    scalar,
    vector: VectorWidth,
    matrix: VectorWidth,
    record_fields: ?*const coral.shaders.Field,
};

const Self = @This();

pub const VectorWidth = enum(u2) {
    @"1",
    @"2",
    @"3",
    @"4",
};

pub const float = &Self{
    .identifier = "float",
    .layout = .scalar,
};

pub const float2 = &Self{
    .identifier = "float2",
    .layout = .{ .vector = .@"2" },
};

pub const float3 = &Self{
    .identifier = "float3",
    .layout = .{ .vector = .@"3" },
};

pub const float4 = &Self{
    .identifier = "float4",
    .layout = .{ .vector = .@"4" },
};

pub const float4x4 = &Self{
    .identifier = "float4x4",
    .layout = .{ .matrix = .@"4" },
};

pub fn hasField(self: Self, field_identifier: []const u8) ?*const coral.shaders.Field {
    return switch (self.layout) {
        .scalar, .matrix, .handle => null,
        .record_fields => |has_field| if (has_field) |field| field.has(field_identifier) else null,

        .vector => |vector_width| switch (vector_width) {
            .@"1" => switch (field_identifier.len) {
                1 => switch (field_identifier[0]) {
                    'x', 'r' => .vector_x,
                    else => null,
                },

                else => null,
            },

            .@"2" => switch (field_identifier.len) {
                1 => switch (field_identifier[0]) {
                    'x', 'r' => .vector_x,
                    'y', 'g' => .vector_y,
                    else => null,
                },

                2 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_xx,
                        'y', 'g' => .vector_xy,
                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_yx,
                        'y', 'g' => .vector_yy,
                        else => null,
                    },

                    else => null,
                },

                else => null,
            },

            .@"3" => switch (field_identifier.len) {
                1 => switch (field_identifier[0]) {
                    'x', 'r' => .vector_x,
                    'y', 'g' => .vector_y,
                    'z', 'b' => .vector_z,
                    else => null,
                },

                2 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_xx,
                        'y', 'g' => .vector_xy,
                        'z', 'b' => .vector_xz,
                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_yx,
                        'y', 'g' => .vector_yy,
                        'z', 'b' => .vector_yz,
                        else => null,
                    },

                    else => null,
                },

                3 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xxx,
                            'y', 'g' => .vector_xxy,
                            'z', 'b' => .vector_xxz,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xyx,
                            'y', 'g' => .vector_xyy,
                            'z', 'b' => .vector_xyz,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xzx,
                            'y', 'g' => .vector_xzy,
                            'z', 'b' => .vector_xzz,
                            else => null,
                        },

                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yxx,
                            'y', 'g' => .vector_yxy,
                            'z', 'b' => .vector_yxz,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yyx,
                            'y', 'g' => .vector_yyy,
                            'z', 'b' => .vector_yyz,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yzx,
                            'y', 'g' => .vector_yzy,
                            'z', 'b' => .vector_yzz,
                            else => null,
                        },

                        else => null,
                    },

                    'z', 'b' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zxx,
                            'y', 'g' => .vector_zxy,
                            'z', 'b' => .vector_zxz,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zyx,
                            'y', 'g' => .vector_zyy,
                            'z', 'b' => .vector_zyz,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zzx,
                            'y', 'g' => .vector_zzy,
                            'z', 'b' => .vector_zzz,
                            else => null,
                        },

                        else => null,
                    },

                    else => null,
                },

                else => null,
            },

            .@"4" => switch (field_identifier.len) {
                1 => switch (field_identifier[0]) {
                    'x', 'r' => .vector_x,
                    'y', 'g' => .vector_y,
                    'z', 'b' => .vector_z,
                    'w', 'a' => .vector_w,
                    else => null,
                },

                2 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_xx,
                        'y', 'g' => .vector_xy,
                        'z', 'b' => .vector_xz,
                        'w', 'a' => .vector_xw,
                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => .vector_yx,
                        'y', 'g' => .vector_yy,
                        'z', 'b' => .vector_yz,
                        'w', 'a' => .vector_yz,
                        else => null,
                    },

                    else => null,
                },

                3 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xxx,
                            'y', 'g' => .vector_xxy,
                            'z', 'b' => .vector_xxz,
                            'w', 'a' => .vector_xxw,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xyx,
                            'y', 'g' => .vector_xyy,
                            'z', 'b' => .vector_xyz,
                            'w', 'a' => .vector_xyw,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_xzx,
                            'y', 'g' => .vector_xzy,
                            'z', 'b' => .vector_xzz,
                            'w', 'a' => .vector_xzw,
                            else => null,
                        },

                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yxx,
                            'y', 'g' => .vector_yxy,
                            'z', 'b' => .vector_yxz,
                            'w', 'a' => .vector_yxw,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yyx,
                            'y', 'g' => .vector_yyy,
                            'z', 'b' => .vector_yyz,
                            'w', 'a' => .vector_yyw,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_yzx,
                            'y', 'g' => .vector_yzy,
                            'z', 'b' => .vector_yzz,
                            'w', 'a' => .vector_yzw,
                            else => null,
                        },

                        else => null,
                    },

                    'z', 'b' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zxx,
                            'y', 'g' => .vector_zxy,
                            'z', 'b' => .vector_zxz,
                            'w', 'a' => .vector_zxw,
                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zyx,
                            'y', 'g' => .vector_zyy,
                            'z', 'b' => .vector_zyz,
                            'w', 'a' => .vector_zyw,
                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => .vector_zzx,
                            'y', 'g' => .vector_zzy,
                            'z', 'b' => .vector_zzz,
                            'w', 'a' => .vector_zzw,
                            else => null,
                        },

                        else => null,
                    },

                    else => null,
                },

                4 => switch (field_identifier[0]) {
                    'x', 'r' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xxxx,
                                'y', 'g' => .vector_xxxy,
                                'z', 'b' => .vector_xxxz,
                                'w', 'a' => .vector_xxxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xxyx,
                                'y', 'g' => .vector_xxyy,
                                'z', 'b' => .vector_xxyz,
                                'w', 'a' => .vector_xxyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xxzx,
                                'y', 'g' => .vector_xxzy,
                                'z', 'b' => .vector_xxzz,
                                'w', 'a' => .vector_xxzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xxwx,
                                'y', 'g' => .vector_xxwy,
                                'z', 'b' => .vector_xxwz,
                                'w', 'a' => .vector_xxww,
                                else => null,
                            },

                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xyxx,
                                'y', 'g' => .vector_xyxy,
                                'z', 'b' => .vector_xyxz,
                                'w', 'a' => .vector_xyxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xyyx,
                                'y', 'g' => .vector_xyyy,
                                'z', 'b' => .vector_xyyz,
                                'w', 'a' => .vector_xyyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xyzx,
                                'y', 'g' => .vector_xyzy,
                                'z', 'b' => .vector_xyzz,
                                'w', 'a' => .vector_xyzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xywx,
                                'y', 'g' => .vector_xywy,
                                'z', 'b' => .vector_xywz,
                                'w', 'a' => .vector_xyww,
                                else => null,
                            },

                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xzxx,
                                'y', 'g' => .vector_xzxy,
                                'z', 'b' => .vector_xzxz,
                                'w', 'a' => .vector_xzxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xzyx,
                                'y', 'g' => .vector_xzyy,
                                'z', 'b' => .vector_xzyz,
                                'w', 'a' => .vector_xzyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xzzx,
                                'y', 'g' => .vector_xzzy,
                                'z', 'b' => .vector_xzzz,
                                'w', 'a' => .vector_xzzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xzwx,
                                'y', 'g' => .vector_xzwy,
                                'z', 'b' => .vector_xzwz,
                                'w', 'a' => .vector_xzww,
                                else => null,
                            },

                            else => null,
                        },

                        'w', 'a' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xwxx,
                                'y', 'g' => .vector_xwxy,
                                'z', 'b' => .vector_xwxz,
                                'w', 'a' => .vector_xwxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xwyx,
                                'y', 'g' => .vector_xwyy,
                                'z', 'b' => .vector_xwyz,
                                'w', 'a' => .vector_xwyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xwzx,
                                'y', 'g' => .vector_xwzy,
                                'z', 'b' => .vector_xwzz,
                                'w', 'a' => .vector_xwzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_xwwx,
                                'y', 'g' => .vector_xwwy,
                                'z', 'b' => .vector_xwwz,
                                'w', 'a' => .vector_xwww,
                                else => null,
                            },

                            else => null,
                        },

                        else => null,
                    },

                    'y', 'g' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yxxx,
                                'y', 'g' => .vector_yxxy,
                                'z', 'b' => .vector_yxxz,
                                'w', 'a' => .vector_yxxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yxyx,
                                'y', 'g' => .vector_yxyy,
                                'z', 'b' => .vector_yxyz,
                                'w', 'a' => .vector_yxyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yxzx,
                                'y', 'g' => .vector_yxzy,
                                'z', 'b' => .vector_yxzz,
                                'w', 'a' => .vector_yxzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yxwx,
                                'y', 'g' => .vector_yxwy,
                                'z', 'b' => .vector_yxwz,
                                'w', 'a' => .vector_yxww,
                                else => null,
                            },

                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yyxx,
                                'y', 'g' => .vector_yyxy,
                                'z', 'b' => .vector_yyxz,
                                'w', 'a' => .vector_yyxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yyyx,
                                'y', 'g' => .vector_yyyy,
                                'z', 'b' => .vector_yyyz,
                                'w', 'a' => .vector_yyyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yyzx,
                                'y', 'g' => .vector_yyzy,
                                'z', 'b' => .vector_yyzz,
                                'w', 'a' => .vector_yyzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yywx,
                                'y', 'g' => .vector_yywy,
                                'z', 'b' => .vector_yywz,
                                'w', 'a' => .vector_yyww,
                                else => null,
                            },

                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yzxx,
                                'y', 'g' => .vector_yzxy,
                                'z', 'b' => .vector_yzxz,
                                'w', 'a' => .vector_yzxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yzyx,
                                'y', 'g' => .vector_yzyy,
                                'z', 'b' => .vector_yzyz,
                                'w', 'a' => .vector_yzyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yzzx,
                                'y', 'g' => .vector_yzzy,
                                'z', 'b' => .vector_yzzz,
                                'w', 'a' => .vector_yzzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_yzwx,
                                'y', 'g' => .vector_yzwy,
                                'z', 'b' => .vector_yzwz,
                                'w', 'a' => .vector_yzww,
                                else => null,
                            },

                            else => null,
                        },

                        'w', 'a' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_ywxx,
                                'y', 'g' => .vector_ywxy,
                                'z', 'b' => .vector_ywxz,
                                'w', 'a' => .vector_ywxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_ywyx,
                                'y', 'g' => .vector_ywyy,
                                'z', 'b' => .vector_ywyz,
                                'w', 'a' => .vector_ywyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_ywzx,
                                'y', 'g' => .vector_ywzy,
                                'z', 'b' => .vector_ywzz,
                                'w', 'a' => .vector_ywzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_ywwx,
                                'y', 'g' => .vector_ywwy,
                                'z', 'b' => .vector_ywwz,
                                'w', 'a' => .vector_ywww,
                                else => null,
                            },

                            else => null,
                        },

                        else => null,
                    },

                    'z', 'b' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zxxx,
                                'y', 'g' => .vector_zxxy,
                                'z', 'b' => .vector_zxxz,
                                'w', 'a' => .vector_zxxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zxyx,
                                'y', 'g' => .vector_zxyy,
                                'z', 'b' => .vector_zxyz,
                                'w', 'a' => .vector_zxyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zxzx,
                                'y', 'g' => .vector_zxzy,
                                'z', 'b' => .vector_zxzz,
                                'w', 'a' => .vector_zxzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zxwx,
                                'y', 'g' => .vector_zxwy,
                                'z', 'b' => .vector_zxwz,
                                'w', 'a' => .vector_zxww,
                                else => null,
                            },

                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zyxx,
                                'y', 'g' => .vector_zyxy,
                                'z', 'b' => .vector_zyxz,
                                'w', 'a' => .vector_zyxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zyyx,
                                'y', 'g' => .vector_zyyy,
                                'z', 'b' => .vector_zyyz,
                                'w', 'a' => .vector_zyyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zyzx,
                                'y', 'g' => .vector_zyzy,
                                'z', 'b' => .vector_zyzz,
                                'w', 'a' => .vector_zyzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zywx,
                                'y', 'g' => .vector_zywy,
                                'z', 'b' => .vector_zywz,
                                'w', 'a' => .vector_zyww,
                                else => null,
                            },

                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zzxx,
                                'y', 'g' => .vector_zzxy,
                                'z', 'b' => .vector_zzxz,
                                'w', 'a' => .vector_zzxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zzyx,
                                'y', 'g' => .vector_zzyy,
                                'z', 'b' => .vector_zzyz,
                                'w', 'a' => .vector_zzyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zzzx,
                                'y', 'g' => .vector_zzzy,
                                'z', 'b' => .vector_zzzz,
                                'w', 'a' => .vector_zzzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zzwx,
                                'y', 'g' => .vector_zzwy,
                                'z', 'b' => .vector_zzwz,
                                'w', 'a' => .vector_zzww,
                                else => null,
                            },

                            else => null,
                        },

                        'w', 'a' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zwxx,
                                'y', 'g' => .vector_zwxy,
                                'z', 'b' => .vector_zwxz,
                                'w', 'a' => .vector_zwxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zwyx,
                                'y', 'g' => .vector_zwyy,
                                'z', 'b' => .vector_zwyz,
                                'w', 'a' => .vector_zwyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zwzx,
                                'y', 'g' => .vector_zwzy,
                                'z', 'b' => .vector_zwzz,
                                'w', 'a' => .vector_zwzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_zwwx,
                                'y', 'g' => .vector_zwwy,
                                'z', 'b' => .vector_zwwz,
                                'w', 'a' => .vector_zwww,
                                else => null,
                            },

                            else => null,
                        },

                        else => null,
                    },

                    'w', 'a' => switch (field_identifier[1]) {
                        'x', 'r' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wxxx,
                                'y', 'g' => .vector_wxxy,
                                'z', 'b' => .vector_wxxz,
                                'w', 'a' => .vector_wxxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wxyx,
                                'y', 'g' => .vector_wxyy,
                                'z', 'b' => .vector_wxyz,
                                'w', 'a' => .vector_wxyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wxzx,
                                'y', 'g' => .vector_wxzy,
                                'z', 'b' => .vector_wxzz,
                                'w', 'a' => .vector_wxzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wxwx,
                                'y', 'g' => .vector_wxwy,
                                'z', 'b' => .vector_wxwz,
                                'w', 'a' => .vector_wxww,
                                else => null,
                            },

                            else => null,
                        },

                        'y', 'g' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wyxx,
                                'y', 'g' => .vector_wyxy,
                                'z', 'b' => .vector_wyxz,
                                'w', 'a' => .vector_wyxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wyyx,
                                'y', 'g' => .vector_wyyy,
                                'z', 'b' => .vector_wyyz,
                                'w', 'a' => .vector_wyyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wyzx,
                                'y', 'g' => .vector_wyzy,
                                'z', 'b' => .vector_wyzz,
                                'w', 'a' => .vector_wyzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wywx,
                                'y', 'g' => .vector_wywy,
                                'z', 'b' => .vector_wywz,
                                'w', 'a' => .vector_wyww,
                                else => null,
                            },

                            else => null,
                        },

                        'z', 'b' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wzxx,
                                'y', 'g' => .vector_wzxy,
                                'z', 'b' => .vector_wzxz,
                                'w', 'a' => .vector_wzxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wzyx,
                                'y', 'g' => .vector_wzyy,
                                'z', 'b' => .vector_wzyz,
                                'w', 'a' => .vector_wzyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wzzx,
                                'y', 'g' => .vector_wzzy,
                                'z', 'b' => .vector_wzzz,
                                'w', 'a' => .vector_wzzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wzwx,
                                'y', 'g' => .vector_wzwy,
                                'z', 'b' => .vector_wzwz,
                                'w', 'a' => .vector_wzww,
                                else => null,
                            },

                            else => null,
                        },

                        'w', 'a' => switch (field_identifier[2]) {
                            'x', 'r' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wwxx,
                                'y', 'g' => .vector_wwxy,
                                'z', 'b' => .vector_wwxz,
                                'w', 'a' => .vector_wwxw,
                                else => null,
                            },

                            'y', 'g' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wwyx,
                                'y', 'g' => .vector_wwyy,
                                'z', 'b' => .vector_wwyz,
                                'w', 'a' => .vector_wwyw,
                                else => null,
                            },

                            'z', 'b' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wwzx,
                                'y', 'g' => .vector_wwzy,
                                'z', 'b' => .vector_wwzz,
                                'w', 'a' => .vector_wwzw,
                                else => null,
                            },

                            'w', 'a' => switch (field_identifier[3]) {
                                'x', 'r' => .vector_wwwx,
                                'y', 'g' => .vector_wwwy,
                                'z', 'b' => .vector_wwwz,
                                'w', 'a' => .vector_wwww,
                                else => null,
                            },

                            else => null,
                        },

                        else => null,
                    },

                    else => null,
                },

                else => null,
            },
        },
    };
}

pub const int = &Self{
    .identifier = "int",
    .layout = .scalar,
};

pub const texture2 = &Self{
    .identifier = "texture2",
    .layout = .handle,
};
