const coral = @import("coral");

const ona = @import("../ona.zig");

const std = @import("std");

action: fn (*ona.App) void,

const Self = @This();

pub fn init(comptime start: anytype) Self {
    const start_fn = switch (@typeInfo(@TypeOf(start))) {
        .@"fn" => |@"fn"| @"fn",
        else => @compileError("parameter `start` must be a function type"),
    };

    if (start_fn.params.len > 1 or (start_fn.params.len == 1 and start_fn.params[0].type != *ona.App)) {
        @compileError(std.fmt.comptimePrint("parameter `start` must accept {s} as the one and only parameter, not {s}", .{
            @typeName(*ona.World),
            @typeName(start_fn.params[0].type.?),
        }));
    }

    const application = struct {
        fn apply(app: *ona.App) void {
            const error_message = std.fmt.comptimePrint("parameter `start` must return void or !void, not {s}", .{
                @typeName(start_fn.return_type.?),
            });

            switch (@typeInfo(start_fn.return_type.?)) {
                .error_union => |error_union| switch (error_union.payload) {
                    void => coral.require(start, .{app}),
                    else => @compileError(error_message),
                },

                .void => start(app),
                else => @compileError(error_message),
            }
        }
    };

    return .{
        .action = application.apply,
    };
}

pub fn initModule(comptime namespace: anytype) Self {
    if (!@hasDecl(namespace, "setup")) {
        @compileError("parameter `namespace` must contain a `setup` declaration");
    }

    return init(namespace.setup);
}

pub fn run(self: Self) void {
    defer {
        coral.heap.traceLeaks();
    }

    var app = coral.require(ona.App.init, .{});

    defer {
        app.deinit();
    }

    self.action(&app);
}

pub fn with(self: Self, comptime other: Self) Self {
    const combination = struct {
        fn apply(app: *ona.App) void {
            other.action(app);
            self.action(app);
        }
    };

    return .{
        .action = combination.apply,
    };
}
